<?php

namespace Drupal\media_pdf_thumbnail\Manager;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\State\StateInterface;
use Drupal\file\FileInterface;
use Drupal\media\MediaInterface;

/**
 * Class MediaPdfThumbnailImageManager.
 *
 * @package Drupal\media_pdf_thumbnail\Manager
 */
class MediaPdfThumbnailImageManager {

  /**
   * MediaPdfThumbnailManager.
   *
   * @var \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager
   */
  protected $mediaPdfThumbnailImagickManager;

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * FileSystem.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * MediaPdfThumbnailManager constructor.
   *
   * @param \Drupal\media_pdf_thumbnail\Manager\MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager
   *   $mediaPdfThumbnailImagickManager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   EntityTypeManager.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   FileSystem.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   * @param \Drupal\Core\State\StateInterface $state
   */
  public function __construct(MediaPdfThumbnailImagickManager $mediaPdfThumbnailImagickManager, EntityTypeManagerInterface $entityTypeManager, FileSystemInterface $fileSystem, ConfigFactoryInterface $configFactory, StateInterface $state) {
    $this->mediaPdfThumbnailImagickManager = $mediaPdfThumbnailImagickManager;
    $this->entityTypeManager = $entityTypeManager;
    $this->fileSystem = $fileSystem;
    $this->configFactory = $configFactory;
    $this->state = $state;
  }

  /**
   * Create pdf thumbnail.
   *
   * @param $entity
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function createThumbnail($entity) {
    if (!$this->supportsEntity($entity)) {
      return;
    }

    $fileEntity = $this->getFileFromMedia($entity);
    $fileEntityInfo = $this->generatePdfImage($fileEntity) ? $this->createThumbnailFileEntity($fileEntity->getFileUri()) : NULL;
    if (!empty($fileEntityInfo)) {
      $this->setImageToMediaThumbnail($entity, $fileEntityInfo);
    }
  }

  /**
   * Gets the file from the media.
   *
   * @param \Drupal\media\MediaInterface $media
   *   A media entity.
   *
   * @return \Drupal\file\FileInterface|null
   *   A file entity object, otherwise null.
   */
  public function getFileFromMedia(MediaInterface $media) {
    $bundle = $media->bundle();
    $config = $this->configFactory->get('media_pdf_thumbnail.bundles.settings');
    $fieldName = $config->get($bundle . '_field');
    if ($fieldName && $media->hasField($fieldName) && !$media->get($fieldName)->isEmpty()) {
      return $media->get($fieldName)->entity;
    }
    return NULL;
  }

  /**
   * Determines if the file is a PDF.
   *
   * @param \Drupal\file\FileInterface $file
   *   A file entity.
   *
   * @return \Drupal\file\FileInterface|null
   *   TRUE if the file is a PDF, otherwise FALSE.
   */
  public function fileIsPdf(FileInterface $file) {
    return $file->getMimeType() === 'application/pdf';
  }

  /**
   * Determines whether the supports a given entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   *
   * @return bool
   *   TRUE if supports that entity, otherwise FALSE.
   */
  public function supportsEntity(EntityInterface $entity) {
    return
      // Only support Media.
      $entity->getEntityTypeId() === 'media'
      // The support must be enable for the bundle.
      && !empty($this->configFactory->get('media_pdf_thumbnail.bundles.settings')->get($entity->bundle() . '_enable'))
      // Must have a file referenced to the media.
      && ($file = $this->getFileFromMedia($entity))
      // File must be a PDF.
      && $this->fileIsPdf($file);
  }

  /**
   * @param $fid
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getFileEntity($fid) {
    return $this->entityTypeManager->getStorage('file')->load($fid);
  }

  /**
   * @param $fileEntity
   *
   * @return bool|string
   */
  protected function generatePdfImage($fileEntity) {
    $fileInfos = $this->getFileInfos($fileEntity);
    if (!empty($fileInfos['source']) && !empty($fileInfos['destination'])) {
      return $this->mediaPdfThumbnailImagickManager->generateImageFromPDF($fileInfos['source'],
        $fileInfos['destination']);
    }
    return NULL;
  }

  /**
   * @param $fileEntity
   *
   * @return array
   */
  protected function getFileInfos($fileEntity) {
    $sourcePath = $fileEntity->getFileUri();
    $destinationPath = $sourcePath . '.jpeg';
    return ['source' => $sourcePath, 'destination' => $destinationPath];
  }

  /**
   * Create file entity.
   *
   * @param string $fileUri
   *   File uri.
   *
   * @return array File entity id.
   *   File entity id.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createThumbnailFileEntity($fileUri) {
    $fileUriArray = explode('/', $fileUri);
    $filenameNoExtension = str_replace('.pdf', '', end($fileUriArray));
    $fileUri = str_replace('.pdf', '.pdf.jpeg', $fileUri);
    $fileEntity = $this->entityTypeManager->getStorage('file')->create([
      'uri' => $fileUri,
      'status' => FILE_STATUS_PERMANENT,
    ]);
    $fileEntity->save();
    return $fileEntity ? [
      'fid' => $fileEntity->id(),
      'filename' => $filenameNoExtension,
    ] : NULL;
  }

  /**
   * @param $entity
   * @param $fileEntityInfo
   *
   */
  protected function setImageToMediaThumbnail($entity, $fileEntityInfo) {
    $entity->set('thumbnail',
      [
        'target_id' => $fileEntityInfo['fid'],
        'alt' => $fileEntityInfo['filename'],
      ]);
    $this->storeFileId($fileEntityInfo['fid']);
  }

  /**
   * @param $fid
   *
   */
  protected function storeFileId($fid) {
    $fids = explode(',', $this->state->get('media_pdf_thumbnail_fids'));
    $fids[] = $fid;
    $this->state->set('media_pdf_thumbnail_fids', implode(',', $fids));
  }

  /**
   * @param $files
   *
   * @return bool
   */
  public function isUsedAsPdfThumbnail($files) {
    $fids = explode(',', $this->state->get('media_pdf_thumbnail_fids'));
    foreach ($files as $file) {
      if (in_array($file->id(), $fids)) {
        return TRUE;
      }
    }
    return FALSE;
  }

}
